Package: PTMods
Type: Package
Version: 1.1.0
Title: Managing Post-Translational Modifications in R
Authors@R: c(person("Laurent", "Gatto", role = "aut",
                    email = "laurent.gatto@uclouvain.be",
                    comment = c(ORCID = "0000-0002-1520-2268")),
             person("Sebastian", "Gibb", role = "aut",
                    email = "mail@sebastiangibb.de",
                    comment = c(ORCID = "0000-0001-7406-4443")),
             person("Guillaume", "Deflandre", role = "cre",
                    email = "guillaume.deflandre@uclouvain.be",
                    comment = c(ORCID = "0009-0008-1257-2416")))
Description: An interface to the community supported database for
             amino acid/protein modifications using mass
             spectrometry.
Depends: R (>= 4.5.0), methods
Suggests: xml2, testthat, knitr, BiocStyle, Biostrings
Enhances: PSMatch
License: GPL-3
VignetteBuilder: knitr
biocViews: Proteomics, MassSpectrometry
BugReports: https://github.com/RforMassSpectrometry/PTMods/issues
URL: https://github.com/RforMassSpectrometry/PTMods
RoxygenNote: 7.3.3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/PTMods
git_branch: devel
git_last_commit: 08ea1e1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:27:25 UTC; biocbuild
Author: Laurent Gatto [aut] (ORCID: <https://orcid.org/0000-0002-1520-2268>),
  Sebastian Gibb [aut] (ORCID: <https://orcid.org/0000-0001-7406-4443>),
  Guillaume Deflandre [cre] (ORCID:
    <https://orcid.org/0009-0008-1257-2416>)
Maintainer: Guillaume Deflandre <guillaume.deflandre@uclouvain.be>
