# Generated by roxygen2: do not edit by hand

export(addFixedModifications)
export(addModifications)
export(addVariableModifications)
export(combineModifications)
export(convertAnnotation)
export(getCanonicalSequence)
export(getModificationsCounts)
importFrom(methods,is)
importFrom(stats,setNames)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,packageVersion)
