#' Managing amino acid modifications for mass spectrometry in R.
#'
#' The `PTMods` package focuses on handling post-translational modifications
#' (PTMs). It distributes PTMs from the Unimod database and allows to convert
#' PTM annotations between multiple formats.
#'
#' @name PTMods-package
#' @references \url{https://github.com/RforMassSpectrometry/PTMods/}
#' @keywords package
#'
#' @importFrom utils packageVersion
"_PACKAGE"
