% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PTM-combine.R
\name{combineModifications}
\alias{combineModifications}
\title{Combines consecutive modifications on amino acids in peptide
sequences.}
\usage{
combineModifications(sequences)
}
\arguments{
\item{sequences}{`character`. A character vector of peptide sequences with
modifications in any format accepted by `convertAnnotation`.
Multiple modifications per amino acid are allowed.}
}
\value{
A character vector of the same length as `sequences` with
consecutive modifications on each amino acid summed into a single
modification. This summed modifications is written in delta mass format.
}
\description{
Combines consecutive modifications on amino acids in peptide
sequences.
}
\examples{
combineModifications("AT[+10][-5]K")
combineModifications("[+10]-[-5]-ATK")
combineModifications("AT[Phospho][UNIMOD:1]K")
combineModifications("EM[+15.9949][-10]EK[+42][-8]")
combineModifications(c("AT[+10][-5]K", "EM[+15.9949][-10]EK"))

}
\author{
Guillaume Deflandre <guillaume.deflandre@uclouvain.be>
}
