% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PTM-utils.R
\name{getCanonicalSequence}
\alias{getCanonicalSequence}
\title{Get canonical sequence from any peptidoform}
\usage{
getCanonicalSequence(x)
}
\arguments{
\item{x}{`character`, ProForma sequence.}
}
\value{
`character`, a `character` vector with sequences cleaned of all
modifications.
}
\description{
Remove any modifications annotation on a sequence. Annotation
from any type (so long as they are specified within brackets "[..]") are
removed.
}
\examples{
getCanonicalSequence(
    c(
        "EM[-15.9949]EVEES[+79.9663]PEK",
        "EK[Acetyl]EVEES[UNIMOD:21]PEK"
    )
)
}
