% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{PTMods-package}
\alias{PTMods}
\alias{PTMods-package}
\title{Managing amino acid modifications for mass spectrometry in R.}
\description{
The `PTMods` package focuses on handling post-translational modifications
(PTMs). It distributes PTMs from the Unimod database and allows to convert
PTM annotations between multiple formats.
}
\references{
\url{https://github.com/RforMassSpectrometry/PTMods/}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/RforMassSpectrometry/PTMods}
  \item Report bugs at \url{https://github.com/RforMassSpectrometry/PTMods/issues}
}

}
\author{
\strong{Maintainer}: Guillaume Deflandre \email{guillaume.deflandre@uclouvain.be} (\href{https://orcid.org/0009-0008-1257-2416}{ORCID})

Authors:
\itemize{
  \item Laurent Gatto \email{laurent.gatto@uclouvain.be} (\href{https://orcid.org/0000-0002-1520-2268}{ORCID})
  \item Sebastian Gibb \email{mail@sebastiangibb.de} (\href{https://orcid.org/0000-0001-7406-4443}{ORCID})
}

}
\keyword{package}
